# Copyright 2000 by Brad Chapman.  All rights reserved.
# This code is part of the Biopython distribution and governed by its
# license.  Please see the LICENSE file that should have been included
# as part of this package.

# stubs generated from the idl
import BioCorba.org__POA.biocorba.seqcore as seqcore__POA
import BioCorba.org.biocorba.seqcore as seqcore

# the class this inherets from
from BioCorba.Server.GnomeMemory.CorbaUnknown import CorbaUnknown

# classes we'll be returning instances of
from CorbaPrimarySeq import CorbaPrimarySeq

# biopython
from Bio import SeqFeature

class CorbaSeqFeature(seqcore__POA.SeqFeature, CorbaUnknown):
    """SeqFeature BioCorba implementation.

    This class inherets from:
    * org__POA.biocorba.seqcore.SeqFeature - The corba stub code
    implementing code for connecting through corba (this is
    generated by the orb implementation.)
    * CorbaUnknown - The memory management interface from which this
    class derives in the idl.
    """
    def __init__(self, seq_feature):
        """Initialize a SeqFeature server object.

        Arguments:
        o seq_feature - The seq_feature which contains the information we'll
        make available.
        """
        CorbaUnknown.__init__(self)
        self._seq_feature = seq_feature

        # a flag to indicate whether or not to return subfeatures.
        # by default we return subfeatures 
        self.return_subfeatures = 1

    def unref(self):
        """Use unref to get rid of unneeded references.
        """
        self._seq_feature = None
        CorbaUnknown.unref(self)

    def type(self):
        """Return the type of the feature.
        """
        return self._seq_feature.type

    def source(self):
        """Return a string with info about the source of the feature.

        Right now Biopython doesn't handle source information so
        we return unknown.
        """
        return "Source <unknown>"
    
    def seq_primary_id(self):
        """Return a string with the primary id of the seq the feature is on.

        Right now we aren't supporting back references like this, so
        we'll just return unknown.
        """
        return "Id <unknown>"

    def start(self):
        """Return the non-fuzzy start position of the feature.
        """
        start_pos = self._seq_feature.location.start
        return int(start_pos.position)

    def end(self):
        """Return the non-fuzzy end position of the feature.
        """
        end_pos = self._seq_feature.location.end
        return int(end_pos.position)

    def strand(self):
        """Return the strand the feature is describing.
        """
        return int(self._seq_feature.strand)

    def qualifiers(self):
        """Return a NameValueSet listing of the qualifiers of the feature.
        """
        qualifiers = []

        qual_keys = self._seq_feature.qualifiers.keys()
        qual_keys.sort()
        for q_key in qual_keys:
            next_qual = \
             seqcore.NameValueSet(q_key, [self._seq_feature.qualifiers[q_key]])
            qualifiers.append(next_qual)
            
        return qualifiers

    def sub_SeqFeatures(self, use_subfeatures):
        """Return a Vector of SeqFeature objects under this feature.
        """
        from Vector import CorbaRestrictedSeqFeatureVector
        
        if self.return_subfeatures:
            seqfeat_vector = \
              CorbaRestrictedSeqFeatureVector(self._seq_feature.sub_features,
                                              1, use_subfeatures)
            return seqfeat_vector.get_object()
        else:
            return None

    def _get_corba_position(self, biopy_position):
        """Convert a biopython position into a corba position.

        Arguments:
        o biopy_position - A biopython position object.
        """
        if isinstance(biopy_position, SeqFeature.ExactPosition):
            return seqcore.SeqFeaturePosition(biopy_position.position, 0,
                                              seqcore.FuzzyTypeCode.EXACT)
        elif isinstance(biopy_position, SeqFeature.WithinPosition):
            return seqcore.SeqFeaturePosition(biopy_position.position,
                                              biopy_position.extension,
                                              seqcore.FuzzyTypeCode.WITHIN)
        elif isinstance(biopy_position, SeqFeature.BetweenPosition):
            return seqcore.SeqFeaturePosition(biopy_position.position,
                                             biopy_position.extension,
                                             seqcore.FuzzyTypeCode.BETWEEN)
        elif isinstance(biopy_position, SeqFeature.BeforePosition):
            return seqcore.SeqFeaturePosition(biopy_position.position, 0,
                                              seqcore.FuzzyTypeCode.BEFORE)
        elif isinstance(biopy_position, SeqFeature.AfterPosition):
            return seqcore.SeqFeaturePosition(biopy_position.position, 0,
                                              seqcore.FuzzyTypeCode.AFTER)
        else:
            raise ValueError("Unexpected position %s" % biopy_position)
            
    def _get_corba_location(self, biopy_location, strand):
        """Convert a biopython location object into a corba location object.

        Arguments:
        o biopy_location - A biopython location object specifying the location.
        o strand - The strand the location is describing.
        """
        corba_start = self._get_corba_position(biopy_location.start)
        corba_end = self._get_corba_position(biopy_location.end)

        return seqcore.SeqFeatureLocation(corba_start, corba_end, strand)

    def locations(self):
        """Return fuzzy locations associated with this feature.
        """
        all_locations = []
        # get the locations for each sub_feature of this feature
        if len(self._seq_feature.sub_features) > 0:
            for feature in self._seq_feature.sub_features:
                corba_location = self._get_corba_location(feature.location,
                                                          feature.strand)

                all_locations.append(corba_location)
                
        # if we don't have subfeatures, just return this feature's location
        else:
            corba_location = \
              self._get_corba_location(self._seq_feature.location,
                                       self._seq_feature.strand)

            all_locations.append(corba_location)

        return all_locations
            
    def PrimarySeq_is_available(self):
        """Return a boolean indicating whether the PrimarySeq is available.

        Never make the PrimarySeq available.
        """
        return 0

    def get_PrimarySeq(self):
        """Return the PrimarySeq corresponding to this feature.

        Right now we don't support this.
        """
        raise seqcore.UnableToProcess("PrimarySeq is not available.")
